interface ProgressData {
    status: "completed" | "failure" | "uploading" | "initialization" | "suspended" | "cancel" | "waiting";
    progress: number;
    total: number;
    file: File;
    result: any;
    createTimestamp: number;
    isCache?: boolean;
}
interface ControllerData {
    pause: () => void;
    resume: () => void;
    remove: () => void;
    _setUploader?: boolean;
}
declare type UploaderData = ProgressData & ControllerData & {
    file: File;
};
interface LargeFileUploaderOptionalOptions {
    slice: boolean;
    sliceSize: number;
    minSlicedFileSize: number;
    numberOfChunks: number;
    retryCountLimit: number;
    multiple: boolean;
    accept: string;
    offlineStorage: boolean;
    spaceName: string;
    immediately: boolean;
    sizeLimit: [number, number] | null;
    numberOfThreads: number;
    maxNumberOfRequest: number;
    gotCache: (cacheList: UploaderData[], uploadDataList: UploaderData[]) => void;
    update: (uploadDataList: UploaderData[]) => void;
    error: (uploadDataList: UploaderData[]) => void;
    success: (data: {
        result: boolean;
        data: UploaderData[];
    }) => void;
    openHash: boolean;
    checkHash: ((hash: string) => Promise<number | true>) | undefined;
}
declare type Upload<T> = (uploadParams: {
    chunks: Blob[] | [File];
    start: number;
    end: number;
    size: number;
    hashSum: string;
}, customParams: T, update: (data: Partial<UploaderData>) => void) => Promise<T>;
interface LargeFileUploaderRequiredOptions<T> {
    upload: Upload<T>;
}
declare type LargeFileUploaderOptions<T> = Partial<LargeFileUploaderOptionalOptions> & LargeFileUploaderRequiredOptions<T>;
declare type FullyLargeFileUploaderOptions<T> = LargeFileUploaderOptionalOptions & LargeFileUploaderRequiredOptions<T>;
declare type CreateUploader = <T>(largeFileUploaderOptions: LargeFileUploaderOptions<T>) => CreateUploaderResponse;
interface CreateUploaderResponse {
    uploadFile: () => void;
    giveFilesToUpload: (files: File[] | FileList) => void;
}

declare const createFileUploader: CreateUploader;

export { ControllerData, CreateUploader, CreateUploaderResponse, FullyLargeFileUploaderOptions, LargeFileUploaderOptionalOptions, LargeFileUploaderOptions, LargeFileUploaderRequiredOptions, ProgressData, Upload, UploaderData, createFileUploader as default };
